import click
import time
from ruamel.yaml import YAML
from speedtest import Speedtest

import netmeter.fping
import netmeter.speedtest
from netmeter.config import print_init
from netmeter.measure import (measure_periodically, measure_once)
import netmeter.transport as transport
from netmeter.config import Configuration

@click.command()
@click.argument('config_file')
def fping(config_file):
    """Use fping to measure network latencies"""
    measure_periodically(config_file, 
                         lambda config: config.get_fping(), 
                         netmeter.fping.fping)


@click.command()
@click.argument('config_file')
def speedtest(config_file):
    """Use speedtest.net client to measure network speeds.
    """
    measure_periodically(config_file, 
                         lambda config: None, 
                         netmeter.speedtest.speedtest)


@click.command()
@click.argument('config_file')
@click.argument('metric_name')
@click.argument('metric_value')
def send(config_file, metric_name, metric_value):
    """Command-line sender for metrics.

    This could be useful for testing a connection, or for calling from 
    a shell script to send a metric established through some external means.
    """
    measure_once(config_file, metric_name, metric_value)


@click.command()
@click.argument('config_file')
def relay(config_file):
    """Command to start a relay service which can read from something like
    MQTT or AMQP, and send to another transport such as MQTT, AMQP, or Carbon.

    Carbon is a transport that will store the metrics read by the relay into
    a GraphiteDB instance.

    MQTT -> MQTT and AMQP -> AMQP are not supported at present, due to 
    the configuration complexity involved.
    """
    config = Configuration(config_file)

    sender = transport.init_sender(config)
    relay = transport.init_relay(config, sender)

    try:
        sender.start()
        relay.start()
    except KeyboardInterrupt:
        print("Shutting down relay")
    finally:
        if relay is not None:
            relay.stop()
        if sender is not None:
            sender.stop()


@click.command()
def init():
    """Print a sample configuration file"""
    print_init()



from ruamel.yaml import YAML

DEFAULT_DELAY = 30

TARGETS = 'targets'
COUNT = 'count'
HOST = 'host'
PORT = 'port'
USER = 'user'
PASSWORD = 'password'
URL = 'url'
QUEUE = 'queue'

DELAY = 'delay'
PREFIX = 'prefix'

SENDER = 'send'
RECEIVER = 'recv'

CARBON = 'carbon'
MQTT = 'mqtt'
AMQP = 'amqp'
CONSOLE = 'console'
FPING = 'fping'

def print_init():
    print(f"""
# Carbon is used to send to GraphiteDB
# Console is used for debugging another sender + bus configuration
{SENDER}: (mqtt | amqp | carbon | console)

# {RECEIVER} is only useful for relay configurations
{RECEIVER}: (mqtt | amqp)

{PREFIX}: mynode                  # Prefix is the metric name prefix, usually node-specific
{DELAY}: 30                       # Speedtest / fping will loop forever with this delay

# If you're configuring a fping reporter, you'll need this section
{FPING}: 
    {TARGETS}: 
    - 192.168.1.1
    - 8.8.8.8
    - github.com

# If you want to relay metrics to a GraphiteDB, you need this section
{CARBON}:
    # Carbon is not needed in 'pure' remote-sender configurations
    # Carbon would be well-suited to measurement / delivery without intermediary bus
    {HOST}: my.graphite-server.com
    {PORT}: 2023

# If you're reporting metrics over a MQTT bus, you need this
{MQTT}:
    {HOST}: m16.cloudmqtt.com
    {PORT}: 20966
    {USER}: blah
    {PASSWORD}: blah

# If you're reporting metrics over a AMQP bus, you need this
{AMQP}:
    {URL}: amqp://<user>:<pass>@my.amqp-server.com/<instance>
    {QUEUE}: some-queue
""")

def get_fping_targets(self, config):
    return config.get(TARGETS) or ['8.8.8.8']

class Configuration(object):
    def __init__(self, config_file):
        with open(config_file) as f:
            self.data = YAML(typ='safe').load(f)
        self.sender = self.data[SENDER]
        self.delay = int(self.data.get(DELAY)) or DEFAULT_DELAY

    def get_prefix(self):
        return self.data[PREFIX]

    def get_relay_type(self):
        return self.data[RECEIVER]

    def get_mqtt(self):
        return self.data[MQTT]

    def get_carbon(self):
        return self.data[CARBON]

    def get_amqp(self):
        return self.data[AMQP]

    def get_fping(self):
        return self.data[FPING]



import time
from netmeter.transport import init_sender
from netmeter.config import Configuration

def measure_once(config_file, metric_name, metric_value):
    sender = None
    try:
        config = Configuration(config_file)

        sender = init_sender(config)
        sender.start()

        now = int(time.time())
        sender.send_raw("%s %s %s" % (metric_name, metric_value, now))
    except KeyboardInterrupt:
        print("Shutting down")
    finally:
        if sender is not None:
            sender.stop()

def measure_periodically(config_file, config_extractor, command):
    sender = None
    try:
        while True:
            config = Configuration(config_file)

            prefix = config.get_prefix()
            command_config = config_extractor(config)

            sender = init_sender(config)
            sender.start()

            # print(f"Starting command: {command} with sender: {sender}, prefix: {prefix}, and config: {command_config}")
            command(sender, prefix, command_config)

            delay = config.delay
            time.sleep(delay)
    except KeyboardInterrupt:
        print("Shutting down")
    finally:
        if sender is not None:
            sender.stop()



from speedtest import Speedtest
import time

def speedtest(sender, prefix, config):
    test = Speedtest()
    test.get_best_server()

    print("Checking download speed...")
    test.download()
    print("Checking upload speed...")
    test.upload()

    now = int(time.time())
    result = test.results.dict()

    sender.send_raw(f"{prefix}.netspeed.download {result['download']} {now}")
    sender.send_raw(f"{prefix}.netspeed.upload {result['upload']} {now}")



import subprocess
import re
import time
from netmeter.config import (TARGETS, COUNT)

FPING_LINE_RE = "\s*[:,=\/]%?\s*"
DEFAULT_COUNT = 20
DEFAULT_TARGETS = ['8.8.8.8']

def fping(sender, prefix, config):
    ping_count = config.get(COUNT) or DEFAULT_COUNT
    command=['/usr/sbin/fping', '-c', str(ping_count), '-q']
    command.extend(config[TARGETS] or DEFAULT_TARGETS)

    records = {}

    print(f"Running '{command}'")
    process = subprocess.Popen(command, stderr=subprocess.PIPE)
    out = process.communicate()[1].decode('utf-8')
    # print(f"Got output:\n{out}\n")

    for line in out.splitlines():
        # line = process.stdout.readline()
        # print(f"Line: '{line}'")
        if line == '':
            continue

        # Sample output:
        # 8.8.8.8     : xmt/rcv/%loss = 5/5/0%, min/avg/max = 75.3/79.5/89.7
        # www.cnn.com : xmt/rcv/%loss = 5/5/0%, min/avg/max = 43.3/46.8/49.5
        #  
        # re.split(FPING_LINE_RE, line) splits into:
        # ['8.8.8.8', 'xmt', 'rcv', '%loss', '5', '5', '0%', 'min', 'avg', 'max', '75.3', '79.5', '89.7']
        #  
        # print(f"Processing output line: '{line}'")
        outer = re.split(FPING_LINE_RE, line.rstrip())
        basename = f"{prefix}.fping.{outer[0].replace('.', '_')}"

        now = int(time.time())

        # print(f"Split line: {outer}")
        if len(outer) > 9:
            for n in range(1,4):
                key = f"{basename}.{outer[n]}"
                val = outer[n+3]
                if val.endswith('%'):
                    val = val[:-1]

                print(f"Added: {key} = {val}")
                sender.send_raw(f"{key} {val} {now}")

            for n in range(7,10):
                key = f"{basename}.{outer[n]}"
                val = outer[n+3]

                print(f"Added: {key} = {val}")
                sender.send_raw(f"{key} {val} {now}")



#!/usr/bin/env python3

from setuptools import setup, find_packages

version='0.3.0'

setup(
    zip_safe=True,
    name='buildchimp-netmeter',
    version=version,
    description="Metrics for network connectivity",
    long_description="Measurements to be pushed into Graphite/Elastic and monitor network connectivity",
    classifiers=[
      "Development Status :: 3 - Alpha",
      "Intended Audience :: Developers",
      "License :: OSI Approved :: GNU General Public License (GPL)",
      "Programming Language :: Python :: 3",
      "Topic :: Utilities",
    ],
    keywords='graphite metrics monitoring network mqtt amqp carbon',
    author='John Casey',
    author_email='jdcasey@commonjava.org',
    url='https://github.com/jdcasey/buildchimp-netmeter',
    license='GPLv3+',
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
    install_requires=[
      'mulay',
      'speedtest-cli',
      'click',
      'ruamel.yaml'
    ],
    test_suite="tests",
    entry_points={
      'console_scripts': [
        'netmeter-fping = netmeter.command:fping',
        'netmeter-speedtest = netmeter.command:speedtest',
        'netmeter-send = netmeter.command:send',
        'netmeter-relay = netmeter.command:relay',
        'netmeter-init = netmeter.command:init'
      ]
    }

)




import mulay.mqtt as mqtt
import mulay.amqp as amqp
import mulay.carbon as carbon
import mulay.console as console

from netmeter.config import (Configuration, AMQP, MQTT, CARBON, CONSOLE)

def init_sender(config):
    if config.sender == MQTT:
        sender = mqtt.Sender(config.get_mqtt())
    elif config.sender == AMQP:
        sender = amqp.Sender(config.get_amqp())
    elif config.sender == CARBON:
        sender = carbon.Sender(config.get_carbon())
    elif config.sender == CONSOLE:
        sender = console.Sender({})
    else:
        raise Exception(f"Unknown sender type: '{config.sender}")

    print(f"Started sender: {config.sender}, instance: {sender}")
    return sender

def init_relay(config, sender):
    relay_type = config.get_relay_type()
    if relay_type == MQTT:
        relay = mqtt.Relay(config.get_mqtt(), sender)
    elif relay_type == AMQP:
        relay = amqp.Relay(config.get_amqp(), sender)
    else:
        raise Exception(f"Unknown relay type: '{relay_type}")

    print(f"Started relay: {relay_type}, instance: {relay}")
    return relay

